# a script to simulate the random dispersal of tree seeds 
# The script loads 2 data files:
#	1) tree_locations.csv: the x and y locations and species of each adult tree
#	2) species_info.csv: the average number of seeds that each species releases
# Doug Jackson and Dave Allen
# R Workshop
# Fall 2010

# define the size of the plot
x_min <- 0 
x_max <- 100
y_min <-0
y_max <- 100

# define other variables
standard_dev<-2

# this is the minimum allowable distance between a seed and an adult; this 
# is meant to represent the average radius of an adult
min_distance<-5

# load the adult tree location data
tree_locations<-read.csv('tree_locations.csv')
attach(tree_locations)

# load the average number of seeds produced by each species
species_info<-read.csv('species_info.csv')
attach(species_info)

# sanity check of the data
print(summary(tree_locations))
cat("\n") # this is an optional command that just prints an empty line
print(summary(species_info))

# a function to find the distance to the closest adult tree
closest_adult<-function(x, y, x_seed, y_seed)
{
	# initialize the variable that keeps track of the minimum distance found so far
	min_dist<-((x_seed-x[1])^2+(y_seed-y[1])^2)^0.5
	
	for(i in 2:length(x))
	{
		temp_dist<-((x_seed-x[i])^2+(y_seed-y[i])^2)^0.5
		if(temp_dist<min_dist)
		{
			min_dist<-temp_dist	
		}
	}
	return(min_dist)
}

# plot the adult trees
plot(x[species=='red'], y[species=='red'], col="red", pch=15,
	xlim=c(x_min, x_max), ylim=c(y_min, y_max), xlab="length", ylab="length")
points(x[species=='blue'], y[species=='blue'], col="blue", pch=15)
points(x[species=='green'], y[species=='green'], col="green", pch=15)
points(x[species=='yellow'], y[species=='yellow'], col="orange", pch=15)

# initialize variables to store seed information
x_seed<-numeric(0)
y_seed<-numeric(0)
species_seed<-character(0)

# loop through all of the adult trees, scattering seeds from each one
for(i in 1:length(x))
{
	# look up the average number of seeds for this species
	avg_seeds<-avg_num_seeds[species_name==species[i]]
	
	# draw the actual number of seeds from a normal distribution
	num_seeds<-round(rnorm(1, mean=avg_seeds, sd=standard_dev))
	
	# scatter the seeds, ensuring that the distance to the closest adult is > min_distance
	num_scattered<-0
	while(num_scattered<num_seeds)
	{
		# choose a random location
		x_rand<-x_min+(x_max-x_min)*runif(1)
		y_rand<-y_min+(y_max-y_min)*runif(1)
		
		# record this location if it isn't too close to an adult
		if(closest_adult(x, y, x_rand, y_rand)>min_distance)
		{
			x_seed<-c(x_seed, x_rand)
			y_seed<-c(y_seed, y_rand)
			species_seed<-c(species_seed, as.character(species[i]))
			num_scattered<-num_scattered+1
		}
	}
}

# add the seedlings to the plot
points(x_seed[species_seed=='red'], y_seed[species_seed=='red'], 
	col="red", cex=0.5, pch=19)
points(x_seed[species_seed=='blue'], y_seed[species_seed=='blue'], 
	col="blue", cex=0.5, pch=19)
points(x_seed[species_seed=='green'], y_seed[species_seed=='green'], 
	col="green", cex=0.5, pch=19)
points(x_seed[species_seed=='yellow'], y_seed[species_seed=='yellow'], 
	col="orange", cex=0.5, pch=19)

# detach the data frames
detach(tree_locations)
detach(species_info)

